Param($clustername, $StaticIPForCluster);

. .\Get-Subnet.ps1;

# For non-terminating errors force execution to stop and throw an exception
$ErrorActionPreference = "Stop";

$ErrorCode_Success = 0;
$ErrorCode_Failed = 1;

Function New-HAGWCluster {
    Param([string] $Name)

    try 
    {
        # Creating cluster and adding first node
        Write-Output "Creating cluster $Name on $env:COMPUTERNAME";
        $cluster = Get-Cluster -Name $Name -ErrorAction Ignore;
        if ($cluster)
        {
            Write-Output "Cluster $Name already exists... proceeding with configuring cluster";
            Return;
        }
        
        $IgnoredNetworks = @();

        # Get all ipv4 static addresses
        $StaticNetworks = Get-Netipaddress | where {$_.PrefixOrigin -eq "Manual" -and $_.addressfamily -eq 2};        
        
        if (!$StaticIPForCluster)
        {
            # No Static IP For Cluster. Verify we have a valid DHCP based Network that belongs to a NetAdapter. 
            # All IPs (IPv4 and IPv6) on this adapter must be DHCP based for management.
            $bValidDHCPNetwork = $False;            
            $DHCPNetworks = Get-Netipaddress | where {($_.PrefixOrigin -eq "Dhcp") -or ($_.PrefixOrigin -eq "RouterAdvertisement")};
            foreach ($DHCPNetwork in $DHCPNetworks)
            {                
                #Find if there is a static IP on the the same interface
                $StaticNetworkFound = Get-Netipaddress | where {$_.InterfaceIndex -eq $DHCPNetwork.InterfaceIndex -and $_.PrefixOrigin -eq "Manual"};
                if ($StaticNetworkFound)
                {
                    continue;
                }

                #Make sure there is a valid netadapter for this DHCP network
                $NetAdapter = get-netadapter | where {$_.InterfaceIndex -eq $DHCPNetwork.InterfaceIndex};
                if ($NetAdapter)
                {
                    $bValidDHCPNetwork = $True;
                    break;
                }
            }

            if (!$bValidDHCPNetwork)
            {
                Write-Output "Failed: No valid DHCP networks were found and no Static IP provided for cluster.";
                Exit $ErrorCode_Failed;                
            }

            # Ignore all static ip subnets since we are creating the cluster based on DHCP
            foreach ($StaticNetwork in $StaticNetworks)
            {
                # Skip the ones that are not associated with an adapter
                $NetAdapter = get-netadapter | where {$_.InterfaceIndex -eq $StaticNetwork.InterfaceIndex};
                if (!$NetAdapter)
                {
                    continue;
                }

                $CurrentSubNetIP = Get-Subnet -IPAddress $StaticNetwork.IPAddress -PrefixLength $StaticNetwork.PrefixLength;

                if (-1 -eq $IgnoredNetworks.IndexOf($CurrentSubNetIP))
                {                   
                   ([array]$IgnoredNetworks) += ([string]$CurrentSubNetIP);
                }
            }
            
            if (0 -eq $IgnoredNetworks.Count)
            {
                Write-Output "Creating cluster using DHCP only. No Static networks to ignore"
                New-Cluster -Name $Name -Node $env:COMPUTERNAME | Out-Null;                
            }
            else
            {
                Write-Output "Creating cluster using DHCP and ignoring Static networks: $IgnoredNetworks"
                New-Cluster -Name $Name -Node $env:COMPUTERNAME -IgnoreNetwork $IgnoredNetworks | Out-Null;
            }
        }
        else
        {
            if (([Net.IPAddress]$StaticIPForCluster).AddressFamily -ne "InterNetworkV6")
            {
                foreach ($StaticNetwork in $StaticNetworks)
                {
                    # Skip the ones that are not associated with an adapter
                    $NetAdapter = get-netadapter | where {$_.InterfaceIndex -eq $StaticNetwork.InterfaceIndex};
                    if (!$NetAdapter)
                    {
                        continue;
                    }

                    $CurrentStaticNetworkSubnet = Get-Subnet -IPAddress $StaticNetwork.IPAddress -PrefixLength $StaticNetwork.PrefixLength;
                    $TempSubnetForClusterIP = Get-Subnet -IPAddress $StaticIPForCluster -PrefixLength $StaticNetwork.PrefixLength;

                    if (($CurrentStaticNetworkSubnet -ne $TempSubnetForClusterIP) -and
                        (-1 -eq $IgnoredNetworks.IndexOf($CurrentStaticNetworkSubnet)))
                    {
                        ([array]$IgnoredNetworks) += ([string]$CurrentStaticNetworkSubnet);
                    }
                }
            }

            if (0 -eq $IgnoredNetworks.Count)
            {
                Write-Output "Creating cluster using Static IPAddress: $StaticIPForCluster"
                New-Cluster -Name $Name -Node $env:COMPUTERNAME -StaticAddress $StaticIPForCluster | Out-Null;                
            }
            else
            {
                Write-Output "Creating cluster using Static IPAddress: $StaticIPForCluster and ignoring Static networks: $IgnoredNetworks"
                New-Cluster -Name $Name -Node $env:COMPUTERNAME -StaticAddress $StaticIPForCluster -IgnoreNetwork $IgnoredNetworks | Out-Null;                
            }
        }
    }
    catch 
    {

        Write-Output "Caught an exception:";
        Write-Output "    Exception Type: $($_.Exception.GetType().FullName)";
        Write-Output "    Exception Message: $($_.Exception.Message)";
        Write-Output "    Exception HResult: $($_.Exception.HResult)";
        Exit $($_.Exception.HResult);
    }
}

try
{
    New-HAGWCluster -Name $clustername;

    # Getting available Microsoft Virtual Disks
    Write-Output "Getting available disks...";
    $disks = Get-Disk | where {$_.FriendlyName -eq "Microsoft Virtual Disk"};
    if ($disks.Count -eq 0)
    {
        Write-Output "No available disks with the friendly name 'Microsoft Virtual Disk'";
        Exit $ErrorCode_Failed;
    }

    # Initializing disks, creating partitions
    Write-Output "===================";
    Write-Output "Creating Partitions";
    Write-Output "===================";
    foreach($disk in $disks) {
      $num = $disk.Number;
      Write-Output "Starting with drive $num";      

      if ($disk.IsOffline -eq $True) {
          Write-Output "Bringing drive online...";
          Set-Disk -Number $num -IsOffline $False;
      }

      if ($disk.PartitionStyle -eq 'RAW') {
          Write-Output "Initializing partition table as MBR...";
          Initialize-Disk -Number $num -PartitionStyle MBR;
      }

      $partition = Get-Partition -DiskNumber $num -ErrorAction Ignore;
      if (!$partition) {
          Write-Output "Creating single partition...";
          New-Partition $num -UseMaximumSize | Out-Null;
      }

      Write-Output "Done with drive $num";
    }
    
    $clusterdisks = Get-ClusterAvailableDisk | Select Name,Number,Size;
    if ($clusterdisks.Count -eq 0) {
        Write-Output "No available cluster disks to process";
    } else {
        # Formating volumes and assigning mount points        
        Write-Output "Formatting disks and assigning drive letters...";
        $letters = 69..90 | ForEach-Object {[char]$_};

        foreach($disk in $clusterdisks) {
            $num = [int]($disk.Name -replace 'Cluster Disk ','');
            $letter = $letters[$num];
            $drivenum = $disk.Number;

            $partition = Get-Partition -DiskNumber $disk.Number -PartitionNumber 1;
            if (!$partition.DriveLetter) {
                Write-Output "Assigning drive letter $letter to Disk $num (Drive $drivenum)";            
                Set-Partition $disk.Number -PartitionNumber 1 -NewDriveLetter $letter | Out-Null;
            }

            $vol = Get-Volume -DriveLetter $letter;
            if ($vol.FileSystem -ne "NTFS") {
                Write-Output "Formatting Cluster Disk $num (Drive $drivenum)";
                Format-Volume -DriveLetter $letter -FileSystem NTFS -Confirm:$false | Out-Null;
            }
        }
    }

    #Adding cluster available disks to cluster
    Write-Output "Adding disks to cluster...";
    Get-ClusterAvailableDisk | Add-ClusterDisk;

    #Setting witness disk. Should be the smallest size disk
    $clusterQuorum = Get-ClusterQuorum;
    if (!$clusterQuorum.QuorumResource) {     

        $smallestSize = [int64]::MaxValue; $smallestDiskName = "";
           
        $resources = Get-WmiObject -namespace root\MSCluster MSCluster_Resource -filter "Type='Physical Disk'";
        foreach ($res in $resources) {
            $clusdisks = $res.GetRelated("MSCluster_Disk");
            foreach ($clusdisk in $clusdisks) {
                if ($clusdisk.Size -lt $smallestSize) {
                   $smallestSize = $clusdisk.Size;
                   $smallestDiskName = $res.Name;
                }                     
            }
        }

        if ($smallestDiskName -eq "") {
            Write-Output "Failed to identify smallest disk to set as a witness disk";
            Exit $ErrorCode_Failed;
        }
        Write-Output "Setting witness disk";
        Set-ClusterQuorum -NodeAndDiskMajority $smallestDiskName;
    }

    #adding "Available Storage" Cluster Disks to Cluster Shared Volumes
    Write-Output "Converting available disks to CSVs ";
    $availableDisks = Get-ClusterGroup -Name "Available Storage" | get-clusterresource;
    if ($availableDisks.Count -eq 0)
    {
        Write-Output "Error: Cannot find available disks to add to Cluster Shared Volumes";
        Exit $ErrorCode_Failed;
    }

    foreach($availableDisk in $availableDisks) {
        Add-ClusterSharedVolume -Name $availableDisk.Name;
    }

    Write-Output "=============================";
    Write-Output "Adding RRAS Cluster Resource ";
    Write-Output "=============================";

    $cr = Get-ClusterResourceType -Name "RAS Cluster Resource" -ErrorAction Ignore;
    if (!$cr) 
    { 
        Add-ClusterResourceType -Name "RAS Cluster Resource" -Dll $env:windir\System32\RasClusterRes.dll;
    }
}
catch
{
   Write-Output "Caught an exception:";
   Write-Output "    Exception Type: $($_.Exception.GetType().FullName)";
   Write-Output "    Exception Message: $($_.Exception.Message)";
   Write-Output "    Exception HResult: $($_.Exception.HResult)";

   Exit $($_.Exception.HResult);
}

Write-Output "Waiting for Cluster...";

$i = 0
$bGetHostAddresses = $False;
while(!$bGetHostAddresses -and ($i -le 100))
{
    try
    {
        Write-Output "Waiting for DNS...";
        $bGetHostAddresses = $([System.Net.Dns]::GetHostAddresses($clusterName));
    }
    catch
    {
       Write-Output "Caught an exception:";
       Write-Output "    Exception Type: $($_.Exception.GetType().FullName)";
       Write-Output "    Exception Message: $($_.Exception.Message)";
       Write-Output "    Exception HResult: $($_.Exception.HResult)";
       [System.Threading.Thread]::Sleep(20000);
    }
    $i++
}

$i = 0
$bGetCluster = $False;
while(!$bGetCluster -and ($i -le 100))
{
    try
    {
        Write-Output "Waiting for Get-Cluster...";
        $bGetCluster = $(Get-Cluster -Name $clusterName)
    }
    catch
    {
       Write-Output "Caught an exception:";
       Write-Output "    Exception Type: $($_.Exception.GetType().FullName)";
       Write-Output "    Exception Message: $($_.Exception.Message)";
       Write-Output "    Exception HResult: $($_.Exception.HResult)";
       [System.Threading.Thread]::Sleep(5000);
    }
    $i++
}

if ($bGetCluster)
{
    Write-Output "Cluster is up"
    Exit $ErrorCode_Success;
}
else
{
    Write-Output "Cluster is not up. Make sure your DNS server"
    Exit $ErrorCode_Failed;
}



